@@ -15,6 +15,7 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
+import abc
 import argparse
 import datetime
 import functools
@@ -50,6 +51,7 @@ class _State:
         mqtt_topic_prefix: str,
         homeassistant_discovery_prefix: str,
         homeassistant_node_id: str,
+        poweroff_delay: datetime.timedelta,
     ) -> None:
         self._mqtt_topic_prefix = mqtt_topic_prefix
         self._homeassistant_discovery_prefix = homeassistant_discovery_prefix
@@ -59,6 +61,7 @@ class _State:
         )  # type: dbus.proxies.Interface
         self._shutdown_lock = None  # type: typing.Optional[dbus.types.UnixFd]
         self._shutdown_lock_mutex = threading.Lock()
+        self.poweroff_delay = poweroff_delay
 
     @property
     def mqtt_topic_prefix(self) -> str:
@@ -73,7 +76,7 @@ class _State:
             assert self._shutdown_lock is None
             # https://www.freedesktop.org/wiki/Software/systemd/inhibit/
             self._shutdown_lock = self._login_manager.Inhibit(
-                "shutdown", "systemctl-mqtt", "Report shutdown via MQTT", "delay",
+                "shutdown", "systemctl-mqtt", "Report shutdown via MQTT", "delay"
             )
             _LOGGER.debug("acquired shutdown inhibitor lock")
 
@@ -90,7 +93,7 @@ class _State:
         return self.mqtt_topic_prefix + "/preparing-for-shutdown"
 
     def _publish_preparing_for_shutdown(
-        self, mqtt_client: paho.mqtt.client.Client, active: bool, block: bool,
+        self, mqtt_client: paho.mqtt.client.Client, active: bool, block: bool
     ) -> None:
         # https://github.com/eclipse/paho.mqtt.python/blob/v1.5.0/src/paho/mqtt/client.py#L1199
         topic = self._preparing_for_shutdown_topic
@@ -98,7 +101,7 @@ class _State:
         payload = systemctl_mqtt._mqtt.encode_bool(active)
         _LOGGER.info("publishing %r on %s", payload, topic)
         msg_info = mqtt_client.publish(
-            topic=topic, payload=payload, retain=True,
+            topic=topic, payload=payload, retain=True
         )  # type: paho.mqtt.client.MQTTMessageInfo
         if not block:
             return
@@ -114,7 +117,7 @@ class _State:
         assert isinstance(active, dbus.Boolean)
         active = bool(active)
         self._publish_preparing_for_shutdown(
-            mqtt_client=mqtt_client, active=active, block=True,
+            mqtt_client=mqtt_client, active=active, block=True
         )
         if active:
             self.release_shutdown_lock()
@@ -132,7 +135,7 @@ class _State:
         )
 
     def publish_preparing_for_shutdown(
-        self, mqtt_client: paho.mqtt.client.Client,
+        self, mqtt_client: paho.mqtt.client.Client
     ) -> None:
         try:
             active = self._login_manager.Get(
@@ -188,17 +191,14 @@ class _State:
         }
         _LOGGER.debug("publishing home assistant config on %s", discovery_topic)
         mqtt_client.publish(
-            topic=discovery_topic, payload=json.dumps(config), retain=True,
+            topic=discovery_topic, payload=json.dumps(config), retain=True
         )
 
 
-class _MQTTAction:
-
-    # pylint: disable=too-few-public-methods
-
-    def __init__(self, name: str, action: typing.Callable) -> None:
-        self.name = name
-        self.action = action
+class _MQTTAction(metaclass=abc.ABCMeta):
+    @abc.abstractmethod
+    def trigger(self, state: _State) -> None:
+        pass  # pragma: no cover
 
     def mqtt_message_callback(
         self,
@@ -213,20 +213,34 @@ class _MQTTAction:
         if message.retain:
             _LOGGER.info("ignoring retained message")
             return
-        _LOGGER.debug("executing action %s (%r)", self.name, self.action)
-        self.action()
-        _LOGGER.debug("completed action %s (%r)", self.name, self.action)
+        _LOGGER.debug("executing action %s", self)
+        self.trigger(state=state)
+        _LOGGER.debug("completed action %s", self)
+
+
+class _MQTTActionSchedulePoweroff(_MQTTAction):
+    def trigger(self, state: _State) -> None:
+        # pylint: disable=protected-access
+        systemctl_mqtt._dbus.schedule_shutdown(
+            action="poweroff", delay=state.poweroff_delay
+        )
+
+    def __str__(self) -> str:
+        return type(self).__name__
+
+
+class _MQTTActionLockAllSessions(_MQTTAction):
+    def trigger(self, state: _State) -> None:
+        # pylint: disable=protected-access
+        systemctl_mqtt._dbus.lock_all_sessions()
+
+    def __str__(self) -> str:
+        return type(self).__name__
 
 
 _MQTT_TOPIC_SUFFIX_ACTION_MAPPING = {
-    "poweroff": _MQTTAction(
-        name="poweroff",
-        action=functools.partial(
-            # pylint: disable=protected-access
-            systemctl_mqtt._dbus.schedule_shutdown,
-            action="poweroff",
-        ),
-    ),
+    "poweroff": _MQTTActionSchedulePoweroff(),
+    "lock-all-sessions": _MQTTActionLockAllSessions(),
 }
 
 
@@ -254,7 +268,7 @@ def _mqtt_on_connect(
             sub=topic, callback=action.mqtt_message_callback
         )
         _LOGGER.debug(
-            "registered MQTT callback for topic %s triggering %r", topic, action.action,
+            "registered MQTT callback for topic %s triggering %s", topic, action
         )
 
 
@@ -266,6 +280,7 @@ def _run(
     mqtt_topic_prefix: str,
     homeassistant_discovery_prefix: str,
     homeassistant_node_id: str,
+    poweroff_delay: datetime.timedelta,
     mqtt_disable_tls: bool = False,
 ) -> None:
     # pylint: disable=too-many-arguments
@@ -277,6 +292,7 @@ def _run(
             mqtt_topic_prefix=mqtt_topic_prefix,
             homeassistant_discovery_prefix=homeassistant_discovery_prefix,
             homeassistant_node_id=homeassistant_node_id,
+            poweroff_delay=poweroff_delay,
         )
     )
     mqtt_client.on_connect = _mqtt_on_connect
@@ -345,7 +361,7 @@ def _main() -> None:
     )
     # https://www.home-assistant.io/docs/mqtt/discovery/#discovery_prefix
     argparser.add_argument(
-        "--homeassistant-discovery-prefix", type=str, default="homeassistant", help=" ",
+        "--homeassistant-discovery-prefix", type=str, default="homeassistant", help=" "
     )
     argparser.add_argument(
         "--homeassistant-node-id",
@@ -354,6 +370,9 @@ def _main() -> None:
         default=systemctl_mqtt._homeassistant.get_default_node_id(),
         help=" ",
     )
+    argparser.add_argument(
+        "--poweroff-delay-seconds", type=float, default=4.0, help=" "
+    )
     args = argparser.parse_args()
     if args.mqtt_port:
         mqtt_port = args.mqtt_port
@@ -389,4 +408,5 @@ def _main() -> None:
         mqtt_topic_prefix=args.mqtt_topic_prefix,
         homeassistant_discovery_prefix=args.homeassistant_discovery_prefix,
         homeassistant_node_id=args.homeassistant_node_id,
+        poweroff_delay=datetime.timedelta(seconds=args.poweroff_delay_seconds),
     )
